#pragma once

#include <ge211.hxx>

namespace widgets {

/// Interface (abstract class) for allowing different classes
/// that are used like `Sprite_set`s.
class ISprite_set
{
public:
    /// Adds the given sprite with the given position, z index,
    /// and transformation.
    virtual void add_sprite(ge211::Sprite const&,
                            ge211::Position,
                            int z,
                            ge211::Transform) = 0;

    /// Adds the given sprite with the given position and optional
    /// z index.
    virtual void add_sprite(ge211::Sprite const&,
                            ge211::Position,
                            int z = 0);
};

/// Wraps a `ge211::Sprite_set` so that we can use it as an
/// `ISprite_set`.
class Sprite_set_adapter : public ISprite_set
{
public:
    explicit Sprite_set_adapter(ge211::Sprite_set&);

    void add_sprite(ge211::Sprite const&,
                    ge211::Position,
                    int z,
                    ge211::Transform) override;

private:
    ge211::Sprite_set& adaptee_;
};

/// The data associated with a sprite placement (used by the next two
/// classes).
struct Placed_sprite
{
    const ge211::Sprite* sprite;
    ge211::Position position;
    int z;
    ge211::Transform transform;
};

/// For testing, a transparent sprite set that just collects all
/// the sprites so we can look at them.
struct Mock_sprite_set : ISprite_set
{
    void add_sprite(ge211::Sprite const&,
                    ge211::Position,
                    int z,
                    ge211::Transform) override;

    std::vector<Placed_sprite> sprites;
};

/// A sprite set that can add its sprites to another sprite set
/// at a specified offset (position and z index).
class Sprite_set_layer : public ISprite_set
{
public:
    void add_sprite(ge211::Sprite const&,
                    ge211::Position,
                    int z,
                    ge211::Transform) override;

    /// Adds the sprites in this layer to `set`, offsetting them
    /// so their lowest z index is the given `z` and translating
    /// them by `offset`. Returns the highest z index used.
    int add_to(ISprite_set& set,
               ge211::Dimensions xyoffset = {0, 0},
               int zoffset = 0) const;

private:
    std::vector<Placed_sprite> sprites_;
    int zmin_ = 0, zmax_ = 0;
};

} // end namespace widgets
